@@ -314,6 +314,7 @@ typedef enum {
     CPL_NONE, CPL_CLEAR, CPL_SAFE, CPL_CONFIDENTIAL, CPL_PRIVATE
 } ChannelProtectionLevel;
 
+int pureftpd_start(int argc, char *argv[], const char *home_directory);
 int safe_write(const int fd, const void *buf_, size_t count);
 #ifdef WITH_TLS
 int secure_safe_write(void * const tls_fd, const void *buf_, size_t count);
@@ -356,7 +357,7 @@ void dostou(void);
 void dofeat(void);
 void domlst(const char * const file);
 void dositetime(void);
-int ul_check_free_space(const char *name);
+int ul_check_free_space(const char *name, const double min_space);
 void mappedtov4(struct sockaddr_storage *ss);
 void disablesignals(void);
 void getnames(void);
@@ -364,6 +365,9 @@ void donlist(char *arg, const int on_ctrlconn, const int opt_l_,
              const int opt_a_, const int split_args);
 void opendata(void);
 void closedata(void);
+void client_fflush(void);
+void client_printf(const char * const format, ...)
+    __attribute__ ((format(printf, 1, 2)));    
 void addreply(const int code, const char * const line, ...)
     __attribute__ ((format(printf, 2, 3)));
 void addreply_noformat(const int code, const char * const line);
@@ -552,10 +556,16 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # define DL_DEFAULT_CHUNK_SIZE_ASCII 32768UL
 #endif
 #ifndef DL_MAX_CHUNK_SIZE
-# define DL_MAX_CHUNK_SIZE (512 * 1024UL)
+# define DL_MAX_CHUNK_SIZE (128 * 1024UL)
 #endif
-#ifndef DL_MMAP_SIZE
-# define DL_MMAP_SIZE (10 * 1024 * 1024UL)
+#ifndef DL_DLMAP_SIZE
+# define DL_DLMAP_SIZE (128 * 1024UL)
+#endif
+#if DL_DEFAULT_CHUNK_SIZE > DL_MAX_CHUNK_SIZE || DL_MIN_CHUNK_SIZE > DL_MAX_CHUNK_SIZE
+# error DL_MAX_CHUNK_SIZE shouldnt be <= DL_MIN_CHUNK_SIZE or <= DL_DEFAULT_CHUNK_SIZE
+#endif
+#if DL_DLMAP_SIZE < DL_MAX_CHUNK_SIZE
+# error DL_DLMAP_SIZE should be >= DL_MAX_CHUNK_SIZE
 #endif
 
 #ifndef UL_MIN_CHUNK_SIZE
@@ -749,6 +759,10 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # define _exit(X) exit(X)
 #endif
 
+#ifndef offsetof
+# define offsetof(type, member) ((size_t) &((type*) NULL)->member)
+#endif    
+    
 #include "bsd-realpath.h"    
 #include "fakechroot.h"
         
