@@ -43,7 +43,7 @@ int main(int argc, char *argv[])
     if (*file == '-' && file[1] == 0) {
         fp = stdin;
     } else {
-        if ((fp = fopen(file, "rt")) == NULL) {
+        if ((fp = fopen(file, "r")) == NULL) {
             perror("Can't open file: ");
             return -1;
         }
@@ -52,7 +52,7 @@ int main(int argc, char *argv[])
         if (instamp >= 0) {
             if (isdigit(c)) {
                 if (instamp < (int) (sizeof timestamp - 1U)) {
-                    timestamp[instamp] = c;
+                    timestamp[instamp] = (char) c;
                     instamp++;
                 }
             } else {
@@ -69,13 +69,16 @@ int main(int argc, char *argv[])
                        tm->tm_sec);
             }
         } else {
-            putchar(c);
+            if (c == '\n' || !ISCTRLCODE(c)) {
+                putchar(c);
+            }
         }
         if (c == '\n') {
             fflush(fp);
             instamp = 0;
         }        
     }
+    fclose(fp);
     
     return 0;
 }
